package com.zanxiang.sdk.service.pay;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.zanxiang.common.enums.OrderStateEnum;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.module.util.DateUtil;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.module.util.pojo.ResultVO;
import com.zanxiang.mybatis.entity.Order;
import com.zanxiang.sdk.constant.RedisKeyConstant;
import com.zanxiang.sdk.domain.bo.PlatformOrderBO;
import com.zanxiang.sdk.domain.bo.ProductPayAttachParamBO;
import com.zanxiang.sdk.domain.bo.ProductPayParamBO;
import com.zanxiang.sdk.domain.dto.GamePayWayDTO;
import com.zanxiang.sdk.domain.dto.PlatformOrderDTO;
import com.zanxiang.sdk.service.IGamePayWayService;
import com.zanxiang.sdk.service.IOrderService;
import com.zanxiang.sdk.util.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.Map;

/**
 * @author : lingfeng
 * @time : 2022-09-28
 * @description : 支付超类
 */
@Slf4j
public abstract class PayBaseService {

    @Lazy
    @Autowired
    public IOrderService orderService;

    @Lazy
    @Autowired
    public IGamePayWayService gamePayWayService;

    @Lazy
    @Autowired
    private RedisUtil<String> redisUtil;

    /**
     * 自定义参数
     */
    protected ProductPayAttachParamBO attach;

    /**
     * 创建订单支付参数
     *
     * @param product : 支付参数
     */
    public Map<String, Object> payCreate(ProductPayParamBO product) {
        try {
            //获取订单信息
            PlatformOrderDTO platformOrderDTO = orderService.getByOrderId(product.getOutTradeNo());
            if (platformOrderDTO == null) {
                throw new BaseException("参数错误, 订单信息不存在");
            }
            Long gameId = platformOrderDTO.getGameId();
            //根据订单中游戏id，获取游戏的支付方式
            GamePayWayDTO gamePayWayDTO = gamePayWayService.getGamePayWay(gameId, product.getPayWay(), product.getPayDevice());
            product.setProductId(platformOrderDTO.getProductId());
            product.setSubject(platformOrderDTO.getProductName());
            product.setGameId(gameId);
            product.setTotalFee(String.valueOf(platformOrderDTO.getAmount()));
            product.setOutTradeNo(platformOrderDTO.getOrderId());
            ProductPayAttachParamBO attachBO = new ProductPayAttachParamBO();
            attachBO.setUserId(product.getUserId());
            attachBO.setOrderId(product.getOutTradeNo());
            attachBO.setPayWay(product.getPayWay());
            attachBO.setGamePayWayId(gamePayWayDTO.getId());
            product.setAttach(attachBO);
            this.attach = attachBO;
            return this.create(product);
        } catch (Exception e) {
            log.info("订单支付参数生成异常, ProductPayParamBO : {} , e: {}", JsonUtil.toString(product), e);
            throw new BaseException("订单支付参数生成异常");
        }
    }

    /**
     * 支付成功逻辑
     *
     * @param orderNo         订单号
     * @param realAmount      真实金额
     * @param merchantOrderNo 订单流水号
     * @param gamePayWayId    具体游戏支付方式id
     * @return : 返回支付状态
     */
    public Boolean paySuccess(String orderNo, String realAmount, String merchantOrderNo, String gamePayWayId) {
        Order order = orderService.getOne(new LambdaQueryWrapper<Order>().eq(Order::getOrderId, orderNo));
        if (order == null) {
            throw new BaseException("支付成功回调, 订单信息不存在");
        }
        PlatformOrderBO bo = new PlatformOrderBO();
        bo.setId(order.getId());
        bo.setOrderId(orderNo);
        bo.setRealAmount(new BigDecimal(realAmount));
        bo.setMerchantOrderNo(merchantOrderNo);
        bo.setStatus(OrderStateEnum.SUCCESS_PAY.getCode());
        bo.setGamePaywayId(gamePayWayId);
        bo.setPayTime(LocalDateTime.now());
        return orderService.pay(bo);
    }

    /**
     * 获取游戏支付配置
     *
     * @param gameId      游戏id
     * @param payWayId    支付方式id
     * @param payDeviceId 支付设备id
     * @return {@link GamePayWayDTO}
     */
    public GamePayWayDTO getPayConfig(Long gameId, Long payWayId, Long payDeviceId) {
        GamePayWayDTO gamePayWayDTO = gamePayWayService.getGamePayWay(gameId, payWayId, payDeviceId);
        this.attach.setGamePayWayId(gamePayWayDTO.getId());
        return gamePayWayDTO;
    }

    /**
     * 订单设置过期时间缓存
     *
     * @param orderId : 订单id
     */
    public void orderExpire(String orderId) {
        //过期时间
        double expire = (double) DateUtil.localDateTimeToMilli(LocalDateTime.now().plusMinutes(5));
        //设置缓存
        redisUtil.addZSet(RedisKeyConstant.ORDER_EXPIRE, orderId, expire);
        log.error("订单添加过期缓存, orderId : {}, expire : {}", orderId, expire);
    }

    /**
     * 支付调起
     *
     * @param product : 调起支付的参数
     * @return : 返回支付参数
     */
    public abstract Map<String, Object> create(ProductPayParamBO product);

    /**
     * 支付异步回调
     *
     * @param request  : 回调请求
     * @param response : 回调返回
     * @return : 返回回调结果
     * @throws IOException : io异常
     */
    public abstract String notify(HttpServletRequest request, HttpServletResponse response) throws IOException;

    /**
     * 支付同步回调
     *
     * @param request : 回调参数
     * @return : 返回回调结果
     */
    public abstract ResultVO synNotify(HttpServletRequest request);

    /**
     * 关闭订单
     *
     * @param platformOrderDTO 平台订单dto
     */
    public abstract void closeOrder(PlatformOrderDTO platformOrderDTO);
}
